from typing import Callable, Dict, List, Optional
from arcanna_mcp_server.environment import MANAGEMENT_API_KEY
from arcanna_mcp_server.constants import INTEGRATION_PARAMETERS_SCHEMA_URL, RESOURCES_CRUD_URL
from arcanna_mcp_server.models.base_resource import BaseResource
from arcanna_mcp_server.models.resource_type import ResourceType
from arcanna_mcp_server.utils.exceptions_handler import handle_exceptions
import requests


def export_tools() -> List[Callable]:
    return [
        integration_parameters_schema,
        upsert_resources,
        get_resources,
        delete_resources,
    ]


@handle_exceptions
async def integration_parameters_schema(integration_type: Optional[str] = None, role: Optional[str] = None) -> Dict:
    """
        Returns the parameters definition for all integrations (or a selected one) as a JSON schema.
        Can be used to determine the types of the integration parameters and determine which are required.
        Some of the parameters given when creating or updating an integration definitions are optional.
        This is specified as such in the response of this endpoint.

        Calling this endpoint with the role parameter returns the type and other specifications of parameters
        needed when creating a job with this type of integration. Integration <-> job parameters are given
        when creating or updating a job using the /api/v2/resources endpoint on the
        job_resource.pipeline_integrations.parameters path. Expected parameters must be specified depending
        on job_resource.pipeline_integrations.role value.
    """
    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }

    params = {}

    if integration_type:
        params["integration_type"] = integration_type

    if role:
        params["role"] = role

    response = requests.get(INTEGRATION_PARAMETERS_SCHEMA_URL, headers=headers, params=params)
    return response.json()


@handle_exceptions
async def upsert_resources(resources: Dict[str, BaseResource], overwrite: Optional[bool] = False) -> Dict:
    """
        Execute a request that creates or updates a set of resources. 
        Resources are given as a JSON document in which on the top level the keys
        are the identifiers of the resources and the values are the resource definitions.

        If the resources don't exist they will be created. If they exists they will be overwritten.
        If the overwrite parameter is set to false an exception will be raised and no change will be made
        to the given set of resources.

        Available resource types that can be created or updated are:
        - 'api_key', 'integration', 'job'.

        Request is handled in a generic way as parameters for each type of available resource can differ.

        The end goal is to be able to create or update a job.

        --------------------------

        The base structure of a resource of type 'api_key' is the following (placeholders are between angular brackets <>):
        {
            '<api_key_identifier>': {
                'properties': {
                    'name': '<api_key_name>',
                },
                'type': 'api_key'
            }
        }

        Placeholder details:

        '<api_key_identifier>' is used for referencing the resource in the request. It is not saved on arcanna in any way.

        '<api_key_name>' represents the name which will be used to save the api key in arcanna.


        The definition of this type of resource is fixed, it doesn't use any dynamic parameters as other type of resources.

        Internally an api key can be used for an integration of type 'External REST API'.
        This type of integration ca be used to send events to arcanna for ingestion.

        When sending events via an integration of type 'External REST API' the value of the api key must be used, not the name.
        Take notice that the value is returned in the response only when creating the api key.
        If the api key with the same title exists it won't return is value again. So the value should be presented to the user or saved locally for later use.

        --------------------------

        The base structure of a resource of type 'integration' is the following (placeholders are between angular brackets <>):
        
        {
            '<integration_identifier>': {
                'properties': {
                    'title': '<integration_title>',
                    'integration_type': '<integration_type>',
                    'parameters': {
                        '<param_key_1>': '<param_value_1>'
                    }
                },
                'type': 'integration'
            }
        }

        Placeholders details:

        '<integration_identifier>': is used for referencing the resource in the request. It is not saved in arcanna.

        '<integration_title>' represents the title of the integration, which will be saved in arcanna.

        '<integration_type>': Available values for integration types and their available parameters
        can be retrieved by calling integration_parameters_schema(). Response is returned as a JSON schema. 
        'integration_type' can be any of the keys from integration_parameters_schema() response on the properties path.

        'parameters': <param_key_1>, <param_value_1> keys and values are dynamic for each type of integration.
        They are described  in the response from  integration_parameters_schema() on the path
        properties.<integration_type>.properties. Being returned as a JSON schema the data type, description,
        mandatory requirement or available valid values for each parameter can be deduced by analysing fields such as:
        'type', 'required' and 'enum' in the response.

        Defining an integration helps defining an external entity to arcanna with the connection details.

        --------------------------

        The base structure of a resource of type job is the following (placeholders are between angular brackets <>):

        {
            '<job_identifier>': {
                'properties': {
                    'title': '<job_title>',
                    'description': '<job_description>',
                    'category': <job_category>',
                    'decision_points': [
                        '<decision_point_name1>', '<decision_point_name2'>
                    ],
                    'advanced_settings: [
                        'custom_labels': [
                            {
                                'name': '<label_name>',
                                'hex_color': '<hex_color>'
                            }
                        ]
                    ],
                    'pipeline_integrations': [
                        {
                            'resource': '<resource_identifier>',
                            'integration_type': '<integration_role>',
                            'enabled': true,
                            'parameters': {
                                '<param_key_1>': '<param_value_1>'
                            }
                        }
                    ],
                    'remove_missing_pipeline_integrations': false
                },
                'type': 'job'
            }
        }

        Placeholder details:

        '<job_identifier>': is used for referencing the resource in the request. It is not saved in arcanna.

        '<job_title>': represents the title of the job, which will be saved in arcanna.

        '<job_description>': description of the job, to be saved in arcanna. it's used just for informative purposes.

        '<job_category>': Arcanna suports the following types of jobs: 'Decision intelligence', 'Event centric decision intelligence'
        and 'Automated root cause analysis'. 'Decision intelligence' does alert triage by tagging alerts with different labels.
        For ease of use this type of jobs groups alerts by the selected feature set. A group of alerts with the same feature set
        is called bucket. By doing this groupping the amount of feedback the user must give is reduced.
        'Event centric decision intelligence' is similar with Decision intelligence in the sense that it does the same
        alert triage. However in this case there is no grouping in buckets. 'Automated root cause analysis' jobs do clustering
        of alerts by a set of fields and within a clusters it splits the events between a root cause and sympthomps. All type of
        jobs are not making any decisions until user feedback is given and a neural network is trained using this feedback.

        By selecting one of the available job categories the 'pipeline_integrations' will have automatically added an integration
        with role 'processor'. In the case of 'Automated root cause analysis jobs' the job will have two 'processor' integrations
        added: one will be doing clustering and one root cause - sympthom classification inside the cluster.

        Predominantly, users will want to create 'Decision intelligence' jobs.

        '<feature_name1>': represents the path of input fields which are used by arcanna in the alert triage procesess.
        Nested fields used dot notation. E.g.: source.ip, source.domain. Only the selected will be used in training and
        decision making.

        '<label_name>', '<hex_color>': - are the labels which are used to split alerts in different categories.
        label_name is the user friendly displayed name. Internally arcanna will use labels ids with the format 
        'label_n' where n is gradually incremented from 0 to n number of labels. 'hex_color' is the hex color code
        used by the label to display this category in Arcanna's UI. A job has a mandatory number of 3 labels.
        If no labels are given the job will be created with default labels for the given 'job_category'. For 
        'Decision intelligence' jobs the default labels will be: 'Escalate', 'Drop', 'Investigate'.
    
        
        '<resource_identifier>' - A job is build from multiple integrations with different roles: 'input', 'processor',
        'output', etc. 
        This field can specify:
            - either the name of the resource. In case this is used the resource must be defined in the same request.
              Take notice that defining the integration in the same request will override the integration if an integration
              with the same title exists in Arcanna.
            - the title of integration as it was saved in arcanna on a previous resource create/update.
            - a query expression where we reference an existing resource eiter by title or by internal id. E.g.:
            "{{integrations(title='Elastic integration')}}", {{integrations(internal_id=1001)}}
        
        '<integration_role>' - Represents the role the integration has in the job. The roles
        in this case can be: 'input', 'processor', 'output', 'enrichment', 'case_creation', 'post_decision'. An integration
        can have multiple roles. For example an Elasticsearch integration can be used both as input and as output.
        All integration types which support a particular role can be discovered by calling integration_parameters_schema()
        with the optional role argument specified.

        'parameters': <param_key_1>, <param_value_1> keys and values are dynamic for each type of integration.
        This are different than integration connection parameters, which are given when defining the integration.
        This represents the parameters that can be used in the combination job <-> integration <-> role.
        To discover the parameters, as well as their data type, description, mandatory requirement or available valid values
        integration_parameters_schema() must be called with the role parameter specified. The parameters are returned as a 
        JSON schema and information about them can be found in the response on path properties.<integration_type>.properties.


        A job allows to define a flow by chaining integrations to collect data, making alert triage, do some post
        decisions and saving the results. Usually a job covers a use case for a class of security threats.

        If the 'pipeline_integrations' is given empty and the job exists it will leave the resulting job.pipeline_integrations
        unchanged.


        Examples:

        1). Decision intelligence job with an External REST API input and an Elasticsearch output.
        
        Request: 
        {
            "Api Key": {
                "properties": {
                    "name": "Api Key from REST API"
                },
                "type": "api_key"
            },
            "Elasticsearch from REST API": {
                "properties": {
                    "title": "Elasticsearch from REST API",
                    "integration_type": "Elasticsearch",
                    "parameters": {
                        "hosts": "192.168.175.175",
                        "password": "elastic",
                        "port": 9200,
                        "schema": "https",
                        "user": "elastic"
                    }
                },
                "type": "integration"
            },
            "Exposer integration from REST API": {
                "properties": {
                    "title": "Exposer integration from REST API",
                    "integration_type": "External REST API",
                    "parameters": {
                        "api_key": "{{api_keys(name='Api Key from REST API')}}",
                        "data_type": "JSON alerts"
                    }
                },
                "type": "integration"
            },
            "Exposer input job from REST API": {
                "properties": {
                    "title": "Exposer input job from REST API",
                    "description": "This is a sample job",
                    "category": "Decision intelligence",
                    "features": [
                        "event.outcome",
                        "event.category"
                    ],
                    "advanced_settings": {
                        "custom_labels": [
                            {
                                "name": "Important",
                                "hex_color": "#a83232"
                            },
                            {
                                "name": "Discard",
                                "hex_color": "#030bff"
                            },
                            {
                                "name": "Potential risk",
                                "hex_color": "#ffcd03"
                            }
                        ]
                    },
                    "pipeline_integrations": [
                        {
                            "resource": "Exposer integration from REST API",
                            "integration_type": "input",
                            "enabled": true,
                            "parameters": {
                                "exposed_rest_api_job_tag": "rest_api_exposer",
                                "max_batch_size": 100
                            }
                        },
                        {
                            "resource": "Elasticsearch from REST API",
                            "integration_type": "output",
                            "enabled": true,
                            "parameters": {}
                        }
                    ]
                },
                "type": "job"
            }
        }

        Response:
        {
            "request": {
                "status": "OK"
            },
            "created": [
                {
                "type": "api_key",
                "resource_name": "Api Key",
                "title": "Api Key from REST API",
                "internal_id": "vGWeqZUB_vwgb0JGPPGE",
                "url": "https://192.168.175.175/api-keys",
                "value": "liNUI2_Ppr5YMag1YD4yuztjH-zlWf7n3C9udVy6QKPGIbisqzJgc-1jdbnbgfi9jCPaaTPj5qkEq5SamNUVCw"
                },
                {
                "type": "integration",
                "resource_name": "Exposer integration from REST API",
                "title": "Exposer integration from REST API",
                "internal_id": 1234,
                "url": "https://192.168.175.175/integrations/1234/edit"
                },
                {
                "type": "integration",
                "resource_name": "Elasticsearch from REST API",
                "title": "Elasticsearch from REST API",
                "internal_id": 1235,
                "url": "https://192.168.175.175/integrations/1235/edit"
                },
                {
                "type": "job",
                "resource_name": "Exposer input job from REST API",
                "title": "Exposer input job from REST API",
                "internal_id": 1316,
                "url": "https://192.168.175.175/use-cases/overview/1316"
                }
            ],
            "updated": []
        }

    """
    try:
        for resource_id, resource in resources.items():
            resources[resource_id] = resource.model_dump()

        body = {
            "resources": resources
        }

        headers = {
            "x-arcanna-api-key": MANAGEMENT_API_KEY,
            "Content-Type": "application/json"
        }

        params = {
            "overwrite": overwrite
        }

        response = requests.post(RESOURCES_CRUD_URL, json=body, headers=headers, params=params)
        response_json = response.json()
    except Exception as e:
        return {"error": str(e)}
    return response_json


async def get_resources(
        resource_type: Optional[ResourceType] = None, title: Optional[str] = None, id: Optional[str] = None) -> Dict:
    """
        Returns a list of resources. Parameters behave as filters.
        If no parameter is provided all available resources from arcanna are returned.

        Parameters:
        -----------
        resource_type : Optional[str]
            Type of resource you want to filter. If not provided it will apply the other filters on all type of resources.
            Available resource types are: api_key, integration or job.
        title : Optional[str]
            The title or name of the searched resource (mutually exclusive with id)
        id : str
            The arcanna internal id of the searched resource (mutually exclusive with title)
    """
    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }

    params = {}

    if resource_type:
        params["resource_type"] = resource_type

    if title:
        params["title"] = title
    elif id:
        params["id"] = id

    response = requests.get(RESOURCES_CRUD_URL, headers=headers, params=params)
    return response.json()


async def delete_resources(
        resource_type: ResourceType, title: Optional[str] = None, id: Optional[str] = None) -> Dict:
    """
        Returns a list of resources. Parameters behave as filters.
        If no parameter is provided all available resources from arcanna are returned.

        Parameters:
        -----------
        resource_type : Optional[str]
            Type of resource for which the delete operation will take place.
            Available resource types are: api_key, integration or job.
        title : Optional[str]
            The title or name of resource that will be deleted (mutually exclusive with id)
        id : str
            The arcanna internal id the resource that will be deleted (mutually exclusive with title)
    """
    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }

    params = {
        "resource_type": resource_type
    }

    if title:
        params["title"] = title
    elif id:
        params["id"] = id

    response = requests.delete(RESOURCES_CRUD_URL, headers=headers, params=params)
    return response.json()



import requests
from typing import List, Callable, Optional
from arcanna_mcp_server.environment import MANAGEMENT_API_KEY
from arcanna_mcp_server.utils.exceptions_handler import handle_exceptions
from arcanna_mcp_server.models.generic_events import QueryEventsRequest, EventModel
from arcanna_mcp_server.models.filters import FilterFieldsRequest, FilterFieldsObject
from arcanna_mcp_server.constants import QUERY_EVENTS_URL, FILTER_FIELDS_URL, EVENT_FEEDBACK_URL_V2


def export_tools() -> List[Callable]:
    return [
        query_arcanna_events,
        get_filter_fields,
        add_feedback_to_event
     ]


@handle_exceptions
async def get_filter_fields(request: FilterFieldsRequest) -> List[FilterFieldsObject]:
    """
    Used to get available fields with available operators and the jobs where the fields are available.
    If neither job_ids nor job_titles are provided, the search will include fields across all jobs.
    To be used within query_arcanna_events tool to get a list of available fields to filter on.

    Parameters:
    -----------
    job_ids : int or list of int or None
        Job IDs to filter on.
    job_titles : str or list of str or None
        Job titles to filter on.
    Returns:
    --------
    list of dictionary
    A dictionary containing job details with the following keys:
        - field_name (str): The field name.
        - available_operators (list of str): A list of available operators for the specified field.
        - available_in_jobs (list of int): A list of jobs where the field is available.
    """
    body = {}

    if request.job_ids:
        body["job_ids"] = request.job_ids

    if request.job_titles:
        body["job_titles"] = request.job_titles

    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }
    response = requests.post(FILTER_FIELDS_URL, json=body, headers=headers)
    return response.json()


@handle_exceptions
async def add_feedback_to_event(job_id: int, event_id: str, label: str, storage_name: Optional[str]) -> dict:
    """
    Provide feedback on a previously ingested event by Arcanna job. The provided feedback will be used to train future AI models
    and make better decisions on new and similar events.

    Parameters:
    -----------
    job_id : int
        Unique identifier for the job.
    event_id : dict
        Unique identifier of the event you want to provide feedback for.
    label: str
        Decision label to be applied for the event. Can be for example Escalate or Drop. Escalate means that the user considers
        the event should be investigated and escalated. Drop means that the event is a false positive and should not
        be investigated.
    storage_name: str or None
        Storage name to be used for feedback. Use only if the job have multiple storages defined.
        If none the feedback will be applied to the latest event with event_id ingested.

    Returns:
    --------
    dict
        A dictionary containing feedback details with the following keys:

        - status (str): Specifies if the feedback was successfully sent or not.
    """
    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }

    if job_id is None:
        raise Exception("Job ID is required.")

    if event_id is None:
        raise Exception("Event ID is required.")

    if label is None:
        raise Exception("Label is required.")

    formatted_url = EVENT_FEEDBACK_URL_V2.format(job_id, event_id) + f'?feedback_label={label}'
    if storage_name:
        formatted_url += f'&storage_name={storage_name}'

    response = requests.put(formatted_url, headers=headers)
    return response.json()


@handle_exceptions
async def query_arcanna_events(request: QueryEventsRequest) -> List[EventModel]:
    """
    Query events filtered by job IDs, job titles, event IDs, or specific filtering criteria.
    If neither job_ids nor job_titles are provided, the search will include events across all jobs.
    Use get_filter_fields tool before to get available fields to apply 'filters' on.
    In case of an internal server error, show the error to the user and do not use any other tool, ask the user how he would like to continue.

    Parameters:
    -----------
    job_ids : int or list of int or None
        Job IDs to filter on.
    job_titles : str or list of str or None
        Job titles to filter on.
    event_ids : str or list of str or None
        Events IDs to filter on.
    decision_points_only : bool or None
         If set to true, only decision points will be included in the response, excluding the full event.
    start_date : str or None
        Start date to filter events newer than this date.
        Date format:
          - ISO 8601 date string (e.g., 'YYYY-MM-DD' or 'YYYY-MM-DDTHH:MM:SS')
          - Elasticsearch time expressions (e.g., 'now-1d', 'now-2h',  'now-30m')
            Examples:
              - 'now-1d' for the last day
              - 'now-2h' for the last two hours
              - 'now-30m' for the last 30 minutes
    end_date : str or None
        End date to filter events older than this date.
        Date format:
          - ISO 8601 date string (e.g., 'YYYY-MM-DD' or 'YYYY-MM-DDTHH:MM:SS')
          - Elasticsearch time expressions (e.g., 'now-1d', 'now-2h',  'now-30m')
            Examples:
              - 'now-1d' for the last day
              - 'now-2h' for the last two hours
              - 'now-30m' for the last 30 minutes
    size : int or None
        Number of events to include in response. If job_ids or job_titles provided it is the number of events per job.
    page : int or None (default: 0)
        Page number, used for pagination. Keep size parameter fixed and increase page size to get more results.
    sort_by_column : str or None
        The field used to sort events. Defaults to the 'timestamp_inference' field; use the default field unless the user specifies a different one.
    sort_order : str or None
        The order in which to sort events by. Defaults to 'desc' order; use the default order unless the user specifies a different one.
    filters : list of dict or None
      Filters to apply to the events returned by the query. If multiple filters are provided, they function as an AND operator between the filters.
      Each filter in list is a dictionary with keys: "field", "operator" and "value"
      - field - the field to apply filters to
      - operator can be: "is", "is not", "is one of", "is not one of", "starts with", "not starts with", "contains", "not contains", "exists", "not exists", "lt", "lte", "gte", "gte"
      - value to filter by, value is omitted for operators "exists" and "not exists" 

        Arcanna fields:
            1. Arcanna decision field = "arcanna.result_label"
            2. Arcanna consensus field = "arcanna.consensus"
            3. Arcanna outlier field flag = "arcanna.outlier_flag"
            4. Arcanna in model status field = "arcanna.knowledge_base_state" or "arcanna.bucket_state"
            5. Arcanna low confidence warning flag field = "attention.low_confidence_score.attention_required"
            6. Arcanna undecided warning flag field = "attention.undecided_consensus.attention_required"

        Predefined filters:
         1. Query outlier events:
            {{
                "filters": [{{
                    "field": "arcanna.outlier_flag",
                    "operator": "is",
                    "value": true
                    }}]
            }}
         2. Query events with low confidence score:
            {{
                "filters": [{{
                    "field": "arcanna.attention.low_confidence_score.attention_required",
                    "operator": "is",
                    "value": true
                    }}]
            }}
         3. Query events with undecided consensus:
            {{
                "filters": [{{
                    "field": "arcanna.attention.undecided_consensus.attention_required",
                    "operator": "is",
                    "value": true
                    }}]
            }}
         4. Query events with any feedback (Event Centric Decision Intelligence job):
            {{
                "filters": [{{
                    "field": "arcanna.knowledge_base_state",
                    "operator": "is",
                    "value": "new"
                    }}]
            }}
         5. Query events without any feedback (Event Centric Decision Intelligence job):
            {{
                "filters": [{{
                    "field": "arcanna.knowledge_base_state",
                    "operator": "is not",
                    "value": "new"
                    }}]
            }}
         6. Query events with any feedback (Decision Intelligence job):
            {{
                "filters": [{{
                    "field": "arcanna.bucket_state",
                    "operator": "is",
                    "value": "new"
                    }}]
            }}
         7. Query events without any feedback (Decision Intelligence job):
            {{
                "filters": [{{
                    "field": "arcanna.bucket_state",
                    "operator": "is not",
                    "value": "new"
                    }}]
            }}
         8. Query events marked as 'Escalate' or 'Investigate' by Arcanna:
            {{
                "filters": [{{
                    "field": "arcanna.result_label",
                    "operator": "is one of",
                    "value": ['Escalate', 'Investigate']
                    }}]
            }}
         9. Query events not marked as 'Drop' or 'Low priority' by Arcanna:
            {{
                "filters": [{{
                    "field": "arcanna.result_label",
                    "operator": "is not one of",
                    "value": ['Drop', 'Low priority']
                    }}]
            }}
         10. Query events with consensus 'Escalate' or 'Drop':
            {{
            "filters": [{{
                "field": "arcanna.consensus",
                "operator": "is one of",
                "value": ['Escalate', 'Drop']
                    }}]
            }}
    
    Returns:
    --------
    list of dictionary
    A dictionary containing job details with the following keys:
        - event_id (str): Unique identifier for the event.
        - job_id (int): Unique identifier of the job where the event was pulled from.
        - job_title (str): Unique identifier for the job where the event was pulled from.
        - decision_points (dict): Dictionary of decision points. Each key in decision_points is a feature used in model training.
        - arcanna (dict or None): Dictionary containing fields added by Arcanna processing.
        - raw_event (dict or None): Dictionary containing the raw event data.
    """

    body = {}

    if request.job_ids:
        body["job_ids"] = request.job_ids

    if request.job_titles:
        body["job_titles"] = request.job_titles

    if request.event_ids:
        body["event_ids"] = request.event_ids

    if request.decision_points_only:
        body["decision_points_only"] = request.decision_points_only

    if request.start_date:
        body["start_date"] = request.start_date

    if request.end_date:
        body["end_date"] = request.end_date

    if request.page:
        body["page"] = request.page

    if request.size:
        body["size"] = request.size

    if request.sort_by_column:
        body["sort_by_column"] = request.sort_by_column

    if request.sort_order:
        body["sort_order"] = request.sort_order

    if request.filters:
        body["filters"] =  request.model_dump().get("filters", [])

    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }
    response = requests.post(QUERY_EVENTS_URL, json=body, headers=headers)
    return response.json()



import requests
from typing import Optional, List, Callable
from arcanna_mcp_server.environment import MANAGEMENT_API_KEY
from arcanna_mcp_server.utils.exceptions_handler import handle_exceptions
from arcanna_mcp_server.constants import CUSTOM_CODE_BLOCK_TEST_URL, CUSTOM_CODE_BLOCK_SAVE_URL


def export_tools() -> List[Callable]:
    return [
        generate_code_agent,
        save_code,
        execute_code
     ]


def compute_python_function(user_query: str) -> str:
    return ""


@handle_exceptions
async def generate_code_agent(user_query: str) -> str:
    """
    Tool to be used when generating code is requested.
    Use this method to generate a code block from a user query.
    After executing this tool ask the user if he wants to execute the code block using the execute_code tool.

    Parameters:
    -----------
    user_query : str
        User query to generate a code block from chat.

    Returns:
    --------
    str
         A Python function that follows the template: def transform(input_record):     # body of the function\n    return input_record
    """
    return compute_python_function(user_query)


@handle_exceptions
async def execute_code(source_code: str, job_id: Optional[int], input_test: dict, env_variables: Optional[list], settings: Optional[dict]) -> dict:
    """
    Send Python function to be executed on Arcanna.
    The function body must follow the template: def transform(input_record):     # body of the function\n    return input_record
    Ask the user before executing this tool for approval, providing the request details.
    In case of an internal server error, show the error to the user and do not use any other tool, ask the user how he would like to continue.

    Parameters:
    -----------
    source_code : str
        The source code to be executed.
    job_id : (int or None)
        Unique identifier for the job.
    input_test : (dict or string of dict)
        Test inputs for the code execution.
    env_variables : (list or None)
        A list of dictionaries containing environment variables with:
        - name (str): Variable name
        - value (str): Variable value
        - is_secret (bool): If the variable contains sensitive data
        - should_encrypt (bool): If the variable should be encrypted
    settings : (dict or None)
        Execution settings including:
        - limits (dict):
            - cpu_time_limit_seconds (int): Maximum CPU execution time
            - memory_limit_mb (int): Memory limit in megabytes

    Returns:
    --------
    dict
        A dictionary containing execution results with the following keys:
        - stdout (str or None): Standard output of execution.
        - stderr (str or None): Standard error output of execution.
        - output_record (str or None): JSON string containing execution records.
    """

    body = {
        "source_code": source_code,
        "input_test": input_test
    }

    if job_id:
        body["job_id"] = job_id

    if env_variables:
        body["env_variables"] = env_variables

    if settings:
        body["settings"] = settings

    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }
    response = requests.post(CUSTOM_CODE_BLOCK_TEST_URL, json=body, headers=headers)
    return response.json()


@handle_exceptions
async def save_code(title: str, description: str, source_code: str, job_id: Optional[int],
                    reprocess_event_id: Optional[str], input_test: dict,
                    env_variables: Optional[list], settings: Optional[dict]) -> dict:
    """
    Code block integration save request. It adds a new code block integration to specified job_id.
    Ask the user before executing this tool for approval, providing the request details.
    In case of an internal server error, show the error to the user and do not use any other tool, ask the user how he would like to continue.

    Parameters:
    -----------
    title : str
        Title of the integration.
    description : str
        A short description of the integration.
    source_code : str
        The source code to be executed.
    job_id : int
        Unique identifier for the job.
    reprocess_event_id : (str or None)
        ID for reprocessing events.
    input_test : (dict or string of dict)
        Test inputs for the code execution.
    env_variables : (list or None)
        A list of dictionaries containing environment variables with:
        - name (str): Variable name
        - value (str): Variable value
        - is_secret (bool): If the variable contains sensitive data
        - should_encrypt (bool): If the variable should be encrypted
    settings : (dict or None)
        Execution settings including:
        - limits (dict):
            - cpu_time_limit_seconds (int): Maximum CPU execution time
            - memory_limit_mb (int): Memory limit in megabytes

    Returns:
    --------
    dict
        A dictionary containing execution results with the following keys:
        - stdout (str or None): Standard output of execution.
        - stderr (str or None): Standard error output of execution.
        - output_record (str or None): JSON string containing execution records.
    """

    body = {
        "description": description,
        "title": title,
        "job_id": job_id,
        "source_code": source_code,
        "input_test": input_test
    }

    if reprocess_event_id:
        body["reprocess_event_id"] = reprocess_event_id

    if env_variables:
        body["env_variables"] = env_variables

    if settings:
        body["settings"] = settings

    headers = {
        "x-arcanna-api-key": MANAGEMENT_API_KEY,
        "Content-Type": "application/json"
    }
    response = requests.post(CUSTOM_CODE_BLOCK_SAVE_URL, json=body, headers=headers)
    return response.json()



from arcanna_mcp_server.environment import ARCANNA_HOST

# healthcheck
HEALTH_CHECK_URL = ARCANNA_HOST + "/api/v2/health/"

# custom code block
CUSTOM_CODE_BLOCK_TEST_URL = ARCANNA_HOST + "/api/v2/custom_code_execution/test"
CUSTOM_CODE_BLOCK_SAVE_URL = ARCANNA_HOST + "/api/v2/custom_code_execution/save"

# resources CRUD operations
RESOURCES_CRUD_URL = ARCANNA_HOST + "/api/v2/resources"

# integration parameters schema
INTEGRATION_PARAMETERS_SCHEMA_URL = ARCANNA_HOST + "/api/v2/resources/integration/parameters/schema"

# events operations
QUERY_EVENTS_URL = ARCANNA_HOST + "/api/v2/events/query"

FILTER_FIELDS_URL = ARCANNA_HOST + "/api/v2/filters/fields"

EVENT_FEEDBACK_URL_V2 = ARCANNA_HOST + "/api/v2/events/{}/{}/feedback"

# jobs
START_JOB_URL = ARCANNA_HOST + "/api/v2/jobs/{}/start"
STOP_JOB_URL = ARCANNA_HOST + "/api/v2/jobs/{}/stop"
TRAIN_JOB_URL = ARCANNA_HOST + "/api/v2/jobs/{}/train"
